
    {
      "requisiteStageRefIds":[""],
      "refId": "master",
      "type": "manualJudgment",
      "name": "Pre-Promote Checkpoint {{ data.app.environment }}",
      "notifications": [],
      "judgmentInputs": [],
      "failPipeline": false,
      {% if data.app.approval_timeout %}
        "stageTimeoutMs": {{ data.app.approval_timeout }},
      {% endif %}
      "instructions": "By approving this S3 content promotion to <strong>{{ data.app.environment }} LATEST</strong>, I certify that:\n<ul>\n<li>I am authorized to perform this change</li>\n<li> To the best of my knowledge and ability, this version of the software is fit for purpose</li>\n<li>I did not develop any aspects of the code or configuration that will be promoted with this approval</li>\n</ul>",
      "comments": "application: ${ trigger.properties.APP_NAME }"
    }
