        {
            "Sid": "DynamoDBReadWrite",
            "Action": [
                "dynamodb:*"
            ],
            "Effect": "Allow",
            "Resource": [
            {%- for table in items %}
                "arn:aws:dynamodb:{{ region }}:{{ account_number }}:table/{{ table }}",
                "arn:aws:dynamodb:{{ region }}:{{ account_number }}:table/{{ table }}/index/*"
                {%- if not loop.last -%}
                ,
                {%- endif -%}
            {% endfor %}
            ]
        },
        {
            "Sid": "DynamoDBStreamRead",
            "Action": [
                "dynamodb:DescribeStream",
                "dynamodb:GetRecords",
                "dynamodb:GetShardIterator",
                "dynamodb:ListStreams"
            ],
            "Effect": "Allow",
            "Resource": [
            {%- for table in items %}
                "arn:aws:dynamodb:{{ region }}:{{ account_number }}:table/{{ table }}/stream/*"
                {%- if not loop.last -%}
                ,
                {%- endif -%}
            {% endfor %}
            ]
        }
