{
    "requisiteStageRefIds": [""],
    "refId": "master",
    "type": "jenkins",
    "comments": "Upload archive file file to s3 bucket in [{{ data.app.environment }}]",
    "name": "Promote S3 [{{ data.app.environment }}]",
    "failPipeline": true,
    "master": "JenkinsCI",
    "job": "s3-promote",
    "parameters": {
      "PROJECT": "{{ data.app.group_name }}",
      "GIT_REPO": "{{ data.app.repo_name }}",
      "ENV": "{{ data.app.environment }}",
      "REGION": "{{ data.app.region }}",
      "OWNER_EMAIL": "{{ data.app.owner_email }}",
      "FUNCTION_NAME": "{{ data.app.function_name }}",
      "ARTIFACT_URL": "${trigger.properties.ARTIFACT_URL}",
      "ARTIFACT_VERSION": "${trigger.properties.ARTIFACT_VERSION}",
      "APP_NAME": "{{ data.app.appname }}"
      }
  }
