{
  "name": "{{ data.app.appname }} [{{ data.app.region }}]",
  {% if data.id %}
  "id": "{{ data.id }}",
  {% endif %}
  "application": "{{ data.app.appname }}",
  "triggers": [
    {% include "pipeline/trigger-jenkins.json.j2" %}
  ],
  "limitConcurrent": false,
  "parallel": true,
  "notifications": {% include "pipeline/pipeline_notifications.json.j2" %},
  "stages": [
    {% if data.app.deploy_type not in ["lambda", "s3", "datapipeline", "stepfunction", "cloudfunction"] %}
      {% include "pipeline/stage-bake.json.j2" %}
    {% endif %}
   ]
}
