{
   "job":[
      {
         "stack":"{{ app_stack }}",
         "detail": "detail",
         "isInternal":{{ isInternal }},
         "credentials":"{{ env }}",
         "region":"{{ region }}",
         "vpcId":"{{ vpc_id }}",
         "healthCheckProtocol":"{{ health_protocol }}",
         "healthCheckPort":{{ health_port }},
         "healthCheckPath":"{{ health_path }}",
         "healthTimeout":{{ health_timeout}},
         "healthInterval":{{ health_interval }},
         "healthyThreshold":{{ healthy_threshold }},
         "unhealthyThreshold":{{ unhealthy_threshold }},
         "idleTimeout": {{ idle_timeout }},
         "accessLog": {{ access_log }},
         "connectionDrainingTimeout": {{ connection_draining_timeout }},
         "regionZones": {{ region_zones }},
         "securityGroups": {{ security_groups }},
         "listeners": {{ listeners }},
         "name":"{{ app_name }}",
         "usePreferredZones":true,
         "subnetType":"{{ subnet_type }}",
         "healthCheck":"{{ hc_string }}",
         "type":"upsertLoadBalancer",
         "availabilityZones": {{ availability_zones }},
         "user":"foremast"
      }
   ],
   "application":"{{ app_name }}",
   "description":"Create Load Balancer: {{ app_name }}-{{ app_stack }}"
}
