{
    "app": {
        "app_description": null,
        "approval_skip": false,
        "approval_timeout": null,
        "archaius_enabled": false,
        "canary": false,
        "custom_tags": {},
        "email": null,
        "eureka_enabled": false,
        "instance_profile": "{{ profile }}",
        "instance_type": "t2.micro",
        "lambda_concurrency_limit": null,
        "lambda_destinations": {},
        "lambda_dlq": {},
        "lambda_environment": {},
        "lambda_filesystems": [],
        "lambda_layers": [],
        "lambda_memory": "128",
        "lambda_role": null,
        "lambda_provisioned_throughput": null,
        "lambda_subnet_count": null,
        "lambda_subnet_purpose": "internal",
        "lambda_timeout": "30",
        "lambda_tracing": {},
        "cloudfunction_timeout": "60",
        "cloudfunction_memory_mb": "128",
        "cloudfunction_max_instances": "0",
        "cloudfunction_vpc_connector": null
    },
    "asg": {
        "hc_type": "ELB",
        "hc_grace_period": 180,
        "app_grace_period": 0,
        "max_inst": 3,
        "min_inst": 1,
        "ssh_keypair": null,
        "subnet_purpose": "internal",
        "enable_public_ips": null,
        "provider_healthcheck": {
            "amazon": false
        },
        "scaling_policy": {},
        "custom_scaling_policies":[],
        "scheduled_actions": []
    },
    "elb": {
        "certificate": null,
        "policies": [],
        "listener_policies": [],
        "backend_policies": [],
        "idle_timeout": null,
        "access_log": {},
        "connection_draining_timeout": null,
        "health": {
            "interval": 20,
            "threshold": 2,
            "timeout": 10,
            "unhealthy_threshold": 5
        },
        "i_port": 8080,
        "i_proto": "HTTP",
        "lb_port": 80,
        "lb_proto": "HTTP",
        "subnet_purpose": "internal",
        "target": "TCP:8080"
    },
    "qe": {
    },
    "regions": {
        "us-east-1": {}
    },
    "deploy_strategy": "highlander",
    "security_group": {
        "description": "Auto-Gen SG for {{ app }}",
        "egress": "0.0.0.0/0",
        "elb_extras": [],
        "ingress": {
        },
        "instance_extras": []
    },
    "dns": {
        "ttl": 60,
        "failover_dns": true,
        "region_specific": true
    },
    "lambda_triggers": [],
    "s3": {
        "shared_bucket_master": false,
        "bucket_name": "",
        "path": "/",
        "bucket_acl": "private",
        "bucket_policy": {},
        "content_metadata": [],
        "cors": {
            "enabled": false,
            "cors_rules": [{
                "cors_headers": [],
                "cors_methods": [],
                "cors_origins": [],
                "cors_expose_headers": [],
                "cors_max_age": 600
            }]
        },
        "encryption": {
            "enabled": false,
            "encryption_rules": [{
                "ApplyServerSideEncryptionByDefault": {
                    "SSEAlgorithm": "AES256"
                }
            }]
        },
        "lifecycle": {
            "enabled": false,
            "lifecycle_rules": [{}]
        },
        "logging": {
            "enabled": false,
            "logging_grants": [],
            "logging_bucket": "",
            "logging_bucket_prefix": "{{ app }}/"
        },
        "notification": {
            "enabled": false,
            "topic_configurations": [{}],
            "queue_configurations": [{}],
            "lambda_configurations": [{}]
        },
        "tagging": {
            "tags": {}
        },
        "versioning": {
            "enabled": false,
            "mfa_delete": "Disabled"
        },
        "website": {
            "enabled": false,
            "index_suffix": "index.html",
            "error_document": "404.html"
        }
    },
    "datapipeline": {
        "description": "",
        "activate_on_deploy": false,
        "json_definition": {}
    },
    "stepfunction": {
        "tracing": {
            "enabled": false
        },
        "logging_configuration": {},
        "statemachine_type": "STANDARD",
        "json_definition": {}
    }
}
